const argv          = require('minimist')(process.argv.slice(2), {
  string: ['q', 'quality', 'd', 'dir', 'f', 'folder']
});
  
const async         = require('async')
const cliProgress   = require('cli-progress')
const jxdl          = require('jxdl-core')

const consts        = require('./lib/const')
const log           = require('./lib/log')
const utils         = require('./lib/utils')

const url           = argv._[0]
  
if (url) {
  if (jxdl.isValidUrl(url)) {
    
    jxdl.getInfo(url, {
      skip: argv.skip || argv.s || 0,
      name: argv.folder || argv.f || null
    }, (err, info) => {
      if (err) log.error('Cannot fetch information')
    
      log.infoList(info, argv)
      
      let task = []
      
      info.songList.forEach(i => {
        task.push(download(i, argv.dir || argv.d, info.name, argv.quality || argv.q))
      })
    
      async.series(task, (err, res) => {
        if (err) console.log(err);
        console.log('\nAll Finished!');
      });
    })
  
  } else {
    log.error('Url not valid')
  }
} else if (argv.v || argv.version) {
  log.welcome()
  log.version()
} else if (argv.h || argv.help) {
  log.help()
} else {
  log.help()
  log.missing()
}

let download = (song, dir=process.cwd(), folder, quality=consts.QUALITY_128) => {
  return (callback) => {
    const bar1 = new cliProgress.Bar({
      format: '[{bar}] {percentage}% | ETA: {eta}s | {value}MB/{total}MB'
    }, cliProgress.Presets.rect);
    
    quality = utils.fixQualityInput(quality)
    let req = jxdl(song.songId, quality);
    
    const filename = song.number + ' - ' + song.title
    log.infoSong(filename)

    req.on('error', (err) => {
      callback(err, null)
    })

    req.on('info', (info) => {
      let length = Math.round(info.size[quality] / 1000000 * 100) / 100
      bar1.start(length, 0);
      
      req.pipe(utils.makeSongStream(dir, folder, filename, info, quality))
    })
    
    req.on('progress', (downloaded) => {
      let dld = Math.round(downloaded / 1000000 * 100) / 100
      bar1.update(dld);
    })

    req.on('end', () => {
      bar1.stop();
      callback(null, 'ok')
    })
  }
}