const fs        = require('fs')
const nodeID3   = require('node-id3')
const sanitize  = require('sanitize-filename')
const request   = require('request')
const path      = require('path')

const consts    = require('./const')

const getExt = (quality) => {
  return (quality == consts.QUALITY_M4A) ? '.m4a' : '.mp3'
}

exports.fixQualityInput = (quality) => {
  switch(quality) {
    case '192': return consts.QUALITY_192
    case '320': return consts.QUALITY_320
    case 'm4a': return consts.QUALITY_M4A
    default: return consts.QUALITY_128
  }
}

exports.checkOrMakeDir = (root, p) => {
  const dir = path.join(root, p)

  if (!fs.existsSync(dir)) {
    fs.mkdirSync(dir);
  }
}

exports.makeSongStream = (dir, folder, filename, info, quality) => {
  folder = sanitize(folder)
  filename = sanitize(filename)
  
  const ext = getExt(quality)
  
  exports.checkOrMakeDir(dir, folder)

  const temp = path.join(dir, folder, '.' + info.id + '.temp')
  const file = path.join(dir, folder, filename + ext)
  
  let stream = fs.createWriteStream(temp)

  stream.on('close', () => {
    fs.renameSync(temp, file)

    const imgFile = path.join(dir, folder, '.' + filename + '.jpg')
    
    exports.writeMp3Tag(file, imgFile, info)
  })

  return stream
}

exports.writeMp3Tag = (file, imgFile, info) => {
  if (info && info.album && info.album.photo) {
    let stream = fs.createWriteStream(imgFile)
  
    stream.on('finish', () => {
      nodeID3.removeTags(file)
      if (nodeID3.write({ title: info.title, artist: info.artist.name, album: info.album.name, image: imgFile }, file)) {
        fs.unlinkSync(imgFile)
      }
    })
    
    request.get(info.album.photo)
      .pipe(stream)
  }
}